/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/

/*
*********************************************************************************************************
*                                              rx32s12xx
*                                           Library Function
*
*                                   Copyright 2021, RX Tech, Corp.
*                                        All Rights Reserved
*
*
* Project      : rx32s12xx
* File         : rx32s12xx_exti.c
* By           : RX_SD_Team
* Description  : Only rx32s12xx
*********************************************************************************************************
*/


/* Includes ------------------------------------------------------------------*/
#include "rx32s12xx_exti.h"

/**
  * @brief  De-initialize the EXTI registers to their default reset values.
  * @retval An ErrorStatus enumeration value:
  *         @arg @ref SUCCESS: EXTI registers are de-initialized
  *         @arg @ref ERROR: not applicable
  */
void EXTI_DeInit(void)
{
  /* Interrupt mask register set to default reset values */
  EXTI_WriteReg(IMR,   0x00000000U);
  /* Event mask register set to default reset values */
  EXTI_WriteReg(EMR,   0x00000000U);
  /* Rising Trigger selection register set to default reset values */
  EXTI_WriteReg(RTSR,  0x00000000U);
  /* Falling Trigger selection register set to default reset values */
  EXTI_WriteReg(FTSR,  0x00000000U);
  /* Software interrupt event register set to default reset values */
  EXTI_WriteReg(SWIER, 0x00000000U);
  /* Pending register clear */
  EXTI_WriteReg(PR,    0x000FFFFFU);

}

/**
  * @brief  Set each @ref EXTI_InitTypeDef field to default value.
  * @param  EXTI_InitStruct Pointer to a @ref EXTI_InitTypeDef structure.
  * @retval None
  */
void EXTI_StructInit(EXTI_InitTypeDef *EXTI_InitStruct)
{
  EXTI_InitStruct->Line           = EXTI_LINE_NONE;
  EXTI_InitStruct->LineCommand    = DISABLE;
  EXTI_InitStruct->Mode           = EXTI_MODE_IT;
  EXTI_InitStruct->Trigger        = EXTI_TRIGGER_FALLING;
}

/**
  * @brief  Initialize the EXTI registers according to the specified parameters in EXTI_InitStruct.
  * @param  EXTI_InitStruct pointer to a @ref EXTI_InitTypeDef structure.
  * @param  Line This parameter can be one of the following values:
  *         @arg @ref EXTI_LINE_0    
  *         @arg @ref EXTI_LINE_1    
  *         @arg @ref EXTI_LINE_2    
  *         @arg @ref EXTI_LINE_3    
  *         @arg @ref EXTI_LINE_4    
  *         @arg @ref EXTI_LINE_5    
  *         @arg @ref EXTI_LINE_6    
  *         @arg @ref EXTI_LINE_7    
  *         @arg @ref EXTI_LINE_16   
  *         @arg @ref EXTI_LINE_17   
  *         @arg @ref EXTI_LINE_21   
  *         @arg @ref EXTI_LINE_22   
  *         @arg @ref EXTI_LINE_A0_31
  * @param  LineCommand This parameter can be one of the following values:
  *         @arg @ref ENABLE
  *         @arg @ref DISABLE  
  * @param  Mode This parameter can be one of the following values:
  *         @arg @ref EXTI_MODE_IT
  *         @arg @ref EXTI_MODE_EVENT 
  *         @arg @ref EXTI_MODE_IT_EVENT 
  * @param  Trigger This parameter can be one of the following values:
  *         @arg @ref EXTI_TRIGGER_RISING
  *         @arg @ref EXTI_TRIGGER_FALLING 
  *         @arg @ref EXTI_TRIGGER_RISING_FALLING 
  * @retval None.
  */
void EXTI_Init(EXTI_InitTypeDef *EXTI_InitStruct)
{
  /* ENABLE LineCommand */
  if (EXTI_InitStruct->LineCommand != DISABLE)
  {
    /* Configure EXTI Lines in range from 0 to 31 */
    if (EXTI_InitStruct->Line != EXTI_LINE_NONE)
    {
      switch (EXTI_InitStruct->Mode)
      {
        case EXTI_MODE_IT:
          /* First Disable Event on provided Lines */
          EXTI_Disable_Event(EXTI_InitStruct->Line);
          /* Then Enable IT on provided Lines */
          EXTI_Enable_IT(EXTI_InitStruct->Line);
          break;
        case EXTI_MODE_EVENT:
          /* First Disable IT on provided Lines */
          EXTI_Disable_IT(EXTI_InitStruct->Line);
          /* Then Enable Event on provided Lines */
          EXTI_Enable_Event(EXTI_InitStruct->Line);
          break;
        case EXTI_MODE_IT_EVENT:
          /* Directly Enable IT & Event on provided Lines */
          EXTI_Enable_IT(EXTI_InitStruct->Line);
          EXTI_Enable_Event(EXTI_InitStruct->Line);
          break;
        default:
          /* First Disable Event on provided Lines */
          EXTI_Disable_Event(EXTI_InitStruct->Line);
          /* Then Enable IT on provided Lines */
          EXTI_Enable_IT(EXTI_InitStruct->Line);
          break;
      }
      if (EXTI_InitStruct->Trigger != EXTI_TRIGGER_NONE)
      {
        switch (EXTI_InitStruct->Trigger)
        {
          case EXTI_TRIGGER_RISING:
            /* First Disable Falling Trigger on provided Lines */
            EXTI_Disable_FallingTrig(EXTI_InitStruct->Line);
            /* Then Enable Rising Trigger on provided Lines */
            EXTI_Enable_RisingTrig(EXTI_InitStruct->Line);
            break;
          case EXTI_TRIGGER_FALLING:
            /* First Disable Rising Trigger on provided Lines */
            EXTI_Disable_RisingTrig(EXTI_InitStruct->Line);
            /* Then Enable Falling Trigger on provided Lines */
            EXTI_Enable_FallingTrig(EXTI_InitStruct->Line);
            break;
          case EXTI_TRIGGER_RISING_FALLING:
            EXTI_Enable_RisingTrig(EXTI_InitStruct->Line);
            EXTI_Enable_FallingTrig(EXTI_InitStruct->Line);
            break;
          default:
            /* First Disable Falling Trigger on provided Lines */
            EXTI_Disable_FallingTrig(EXTI_InitStruct->Line);
            /* Then Enable Rising Trigger on provided Lines */
            EXTI_Enable_RisingTrig(EXTI_InitStruct->Line);
            break;
        }
      }
    }
  }
  /* DISABLE LineCommand */
  else
  {
    /* De-configure EXTI Lines in range from 0 to 31 */
    EXTI_Disable_IT(EXTI_InitStruct->Line);
    EXTI_Disable_Event(EXTI_InitStruct->Line);
  }
}


/**
  * @brief  Enable ExtiLine Interrupt request for Lines in range 0 to 31
  * @note The reset value for the direct or internal lines (see RM)
  *       is set to 1 in order to enable the interrupt by default.
  *       Bits are set automatically at Power on.
  * @rmtoll EXTI_Enable_IT
  * @param  ExtiLine This parameter can be one of the following values:
  *         @arg @ref EXTI_LINE_0    
  *         @arg @ref EXTI_LINE_1    
  *         @arg @ref EXTI_LINE_2    
  *         @arg @ref EXTI_LINE_3    
  *         @arg @ref EXTI_LINE_4    
  *         @arg @ref EXTI_LINE_5    
  *         @arg @ref EXTI_LINE_6    
  *         @arg @ref EXTI_LINE_7    
  *         @arg @ref EXTI_LINE_16   
  *         @arg @ref EXTI_LINE_17   
  *         @arg @ref EXTI_LINE_21   
  *         @arg @ref EXTI_LINE_22   
  *         @arg @ref EXTI_LINE_A0_31
  * @note   Please check each device line mapping for EXTI Line availability
  * @retval None
  */
void EXTI_Enable_IT(uint32_t ExtiLine)
{
  SET_BIT(EXTI->IMR, ExtiLine);
}

/**
  * @brief  Disable ExtiLine Interrupt request for Lines in range 0 to 31
  * @note The reset value for the direct or internal lines (see RM)
  *       is set to 1 in order to enable the interrupt by default.
  *       Bits are set automatically at Power on.
  * @rmtoll EXTI_Disable_IT
  * @param  ExtiLine This parameter can be one of the following values:
  *         @arg @ref EXTI_LINE_0    
  *         @arg @ref EXTI_LINE_1    
  *         @arg @ref EXTI_LINE_2    
  *         @arg @ref EXTI_LINE_3    
  *         @arg @ref EXTI_LINE_4    
  *         @arg @ref EXTI_LINE_5    
  *         @arg @ref EXTI_LINE_6    
  *         @arg @ref EXTI_LINE_7    
  *         @arg @ref EXTI_LINE_16   
  *         @arg @ref EXTI_LINE_17   
  *         @arg @ref EXTI_LINE_21   
  *         @arg @ref EXTI_LINE_22   
  *         @arg @ref EXTI_LINE_A0_31
  * @note   Please check each device line mapping for EXTI Line availability
  * @retval None
  */
void EXTI_Disable_IT(uint32_t ExtiLine)
{
  CLEAR_BIT(EXTI->IMR, ExtiLine);
}


/**
  * @brief  Indicate if ExtiLine Interrupt request is enabled for Lines in range 0 to 31
  * @note The reset value for the direct or internal lines (see RM)
  *       is set to 1 in order to enable the interrupt by default.
  *       Bits are set automatically at Power on.
  * @rmtoll EXTI_Get_IT
  * @param  ExtiLine This parameter can be one of the following values:
  *         @arg @ref EXTI_LINE_0    
  *         @arg @ref EXTI_LINE_1    
  *         @arg @ref EXTI_LINE_2    
  *         @arg @ref EXTI_LINE_3    
  *         @arg @ref EXTI_LINE_4    
  *         @arg @ref EXTI_LINE_5    
  *         @arg @ref EXTI_LINE_6    
  *         @arg @ref EXTI_LINE_7    
  *         @arg @ref EXTI_LINE_16   
  *         @arg @ref EXTI_LINE_17   
  *         @arg @ref EXTI_LINE_21   
  *         @arg @ref EXTI_LINE_22   
  *         @arg @ref EXTI_LINE_A0_31
  * @note   Please check each device line mapping for EXTI Line availability
  * @retval State of bit (1 or 0).
  */
uint32_t EXTI_Get_IT(uint32_t ExtiLine)
{
  return (READ_BIT(EXTI->IMR, ExtiLine) == (ExtiLine));
}

/**
  * @brief  Enable ExtiLine Event request for Lines in range 0 to 31
  * @rmtoll EXTI_Enable_Event
  * @param  ExtiLine This parameter can be one of the following values:
  *         @arg @ref EXTI_LINE_0    
  *         @arg @ref EXTI_LINE_1    
  *         @arg @ref EXTI_LINE_2    
  *         @arg @ref EXTI_LINE_3    
  *         @arg @ref EXTI_LINE_4    
  *         @arg @ref EXTI_LINE_5    
  *         @arg @ref EXTI_LINE_6    
  *         @arg @ref EXTI_LINE_7    
  *         @arg @ref EXTI_LINE_16   
  *         @arg @ref EXTI_LINE_17   
  *         @arg @ref EXTI_LINE_21   
  *         @arg @ref EXTI_LINE_22   
  *         @arg @ref EXTI_LINE_A0_31
  * @note   Please check each device line mapping for EXTI Line availability
  * @retval None
  */
void EXTI_Enable_Event(uint32_t ExtiLine)
{
  SET_BIT(EXTI->EMR, ExtiLine);
}


/**
  * @brief  Disable ExtiLine Event request for Lines in range 0 to 31
  * @rmtoll EXTI_Disable_Event
  * @param  ExtiLine This parameter can be one of the following values:
  *         @arg @ref EXTI_LINE_0    
  *         @arg @ref EXTI_LINE_1    
  *         @arg @ref EXTI_LINE_2    
  *         @arg @ref EXTI_LINE_3    
  *         @arg @ref EXTI_LINE_4    
  *         @arg @ref EXTI_LINE_5    
  *         @arg @ref EXTI_LINE_6    
  *         @arg @ref EXTI_LINE_7    
  *         @arg @ref EXTI_LINE_16   
  *         @arg @ref EXTI_LINE_17   
  *         @arg @ref EXTI_LINE_21   
  *         @arg @ref EXTI_LINE_22   
  *         @arg @ref EXTI_LINE_A0_31
  * @note   Please check each device line mapping for EXTI Line availability
  * @retval None
  */
void EXTI_Disable_Event(uint32_t ExtiLine)
{
  CLEAR_BIT(EXTI->EMR, ExtiLine);
}


/**
  * @brief  Indicate if ExtiLine Event request is enabled for Lines in range 0 to 31
  * @rmtoll EXTI_Get_Event
  * @param  ExtiLine This parameter can be one of the following values:
  *         @arg @ref EXTI_LINE_0    
  *         @arg @ref EXTI_LINE_1    
  *         @arg @ref EXTI_LINE_2    
  *         @arg @ref EXTI_LINE_3    
  *         @arg @ref EXTI_LINE_4    
  *         @arg @ref EXTI_LINE_5    
  *         @arg @ref EXTI_LINE_6    
  *         @arg @ref EXTI_LINE_7    
  *         @arg @ref EXTI_LINE_16   
  *         @arg @ref EXTI_LINE_17   
  *         @arg @ref EXTI_LINE_21   
  *         @arg @ref EXTI_LINE_22   
  *         @arg @ref EXTI_LINE_A0_31
  * @note   Please check each device line mapping for EXTI Line availability
  * @retval State of bit (1 or 0).
  */
uint32_t EXTI_Get_Event(uint32_t ExtiLine)
{
  return (READ_BIT(EXTI->EMR, ExtiLine) == (ExtiLine));

}

/**
  * @brief  Enable ExtiLine Rising Edge Trigger for Lines in range 0 to 31
  * @note The configurable wakeup lines are edge-triggered. No glitch must be
  *       generated on these lines. If a rising edge on a configurable interrupt
  *       line occurs during a write operation in the EXTI_RTSR register, the
  *       pending bit is not set.
  *       Rising and falling edge triggers can be set for
  *       the same interrupt line. In this case, both generate a trigger
  *       condition.
  * @rmtoll EXTI_Enable_RisingTrig
  * @param  ExtiLine This parameter can be a combination of the following values:
  *         @arg @ref EXTI_LINE_0    
  *         @arg @ref EXTI_LINE_1    
  *         @arg @ref EXTI_LINE_2    
  *         @arg @ref EXTI_LINE_3    
  *         @arg @ref EXTI_LINE_4    
  *         @arg @ref EXTI_LINE_5    
  *         @arg @ref EXTI_LINE_6    
  *         @arg @ref EXTI_LINE_7    
  *         @arg @ref EXTI_LINE_16   
  *         @arg @ref EXTI_LINE_17   
  *         @arg @ref EXTI_LINE_21   
  *         @arg @ref EXTI_LINE_22   
  *         @arg @ref EXTI_LINE_A0_31
  * @note   Please check each device line mapping for EXTI Line availability
  * @retval None
  */
void EXTI_Enable_RisingTrig(uint32_t ExtiLine)
{
  SET_BIT(EXTI->RTSR, ExtiLine);
}


/**
  * @brief  Disable ExtiLine Rising Edge Trigger for Lines in range 0 to 31
  * @note The configurable wakeup lines are edge-triggered. No glitch must be
  *       generated on these lines. If a rising edge on a configurable interrupt
  *       line occurs during a write operation in the EXTI_RTSR register, the
  *       pending bit is not set.
  *       Rising and falling edge triggers can be set for
  *       the same interrupt line. In this case, both generate a trigger
  *       condition.
  * @rmtoll EXTI_Disable_RisingTrig
  * @param  ExtiLine This parameter can be a combination of the following values:
  *         @arg @ref EXTI_LINE_0    
  *         @arg @ref EXTI_LINE_1    
  *         @arg @ref EXTI_LINE_2    
  *         @arg @ref EXTI_LINE_3    
  *         @arg @ref EXTI_LINE_4    
  *         @arg @ref EXTI_LINE_5    
  *         @arg @ref EXTI_LINE_6    
  *         @arg @ref EXTI_LINE_7    
  *         @arg @ref EXTI_LINE_16   
  *         @arg @ref EXTI_LINE_17   
  *         @arg @ref EXTI_LINE_21   
  *         @arg @ref EXTI_LINE_22   
  *         @arg @ref EXTI_LINE_A0_31
  * @note   Please check each device line mapping for EXTI Line availability
  * @retval None
  */
void EXTI_Disable_RisingTrig(uint32_t ExtiLine)
{
  CLEAR_BIT(EXTI->RTSR, ExtiLine);
}


/**
  * @brief  Check if rising edge trigger is enabled for Lines in range 0 to 31
  * @rmtoll EXTI_Get_RisingTrig
  * @param  ExtiLine This parameter can be a combination of the following values:
  *         @arg @ref EXTI_LINE_0    
  *         @arg @ref EXTI_LINE_1    
  *         @arg @ref EXTI_LINE_2    
  *         @arg @ref EXTI_LINE_3    
  *         @arg @ref EXTI_LINE_4    
  *         @arg @ref EXTI_LINE_5    
  *         @arg @ref EXTI_LINE_6    
  *         @arg @ref EXTI_LINE_7    
  *         @arg @ref EXTI_LINE_16   
  *         @arg @ref EXTI_LINE_17   
  *         @arg @ref EXTI_LINE_21   
  *         @arg @ref EXTI_LINE_22   
  *         @arg @ref EXTI_LINE_A0_31
  * @note   Please check each device line mapping for EXTI Line availability
  * @retval State of bit (1 or 0).
  */
uint32_t EXTI_Get_RisingTrig(uint32_t ExtiLine)
{
  return (READ_BIT(EXTI->RTSR, ExtiLine) == (ExtiLine));
}

/**
  * @brief  Enable ExtiLine Falling Edge Trigger for Lines in range 0 to 31
  * @note The configurable wakeup lines are edge-triggered. No glitch must be
  *       generated on these lines. If a falling edge on a configurable interrupt
  *       line occurs during a write operation in the EXTI_FTSR register, the
  *       pending bit is not set.
  *       Rising and falling edge triggers can be set for
  *       the same interrupt line. In this case, both generate a trigger
  *       condition.
  * @rmtoll EXTI_Enable_FallingTrig
  * @param  ExtiLine This parameter can be a combination of the following values:
  *         @arg @ref EXTI_LINE_0    
  *         @arg @ref EXTI_LINE_1    
  *         @arg @ref EXTI_LINE_2    
  *         @arg @ref EXTI_LINE_3    
  *         @arg @ref EXTI_LINE_4    
  *         @arg @ref EXTI_LINE_5    
  *         @arg @ref EXTI_LINE_6    
  *         @arg @ref EXTI_LINE_7    
  *         @arg @ref EXTI_LINE_16   
  *         @arg @ref EXTI_LINE_17   
  *         @arg @ref EXTI_LINE_21   
  *         @arg @ref EXTI_LINE_22   
  *         @arg @ref EXTI_LINE_A0_31
  * @note   Please check each device line mapping for EXTI Line availability
  * @retval None
  */
void EXTI_Enable_FallingTrig(uint32_t ExtiLine)
{
  SET_BIT(EXTI->FTSR, ExtiLine);
}


/**
  * @brief  Disable ExtiLine Falling Edge Trigger for Lines in range 0 to 31
  * @note The configurable wakeup lines are edge-triggered. No glitch must be
  *       generated on these lines. If a Falling edge on a configurable interrupt
  *       line occurs during a write operation in the EXTI_FTSR register, the
  *       pending bit is not set.
  *       Rising and falling edge triggers can be set for the same interrupt line.
  *       In this case, both generate a trigger condition.
  * @rmtoll EXTI_Disable_FallingTrig
  * @param  ExtiLine This parameter can be a combination of the following values:
  *         @arg @ref EXTI_LINE_0    
  *         @arg @ref EXTI_LINE_1    
  *         @arg @ref EXTI_LINE_2    
  *         @arg @ref EXTI_LINE_3    
  *         @arg @ref EXTI_LINE_4    
  *         @arg @ref EXTI_LINE_5    
  *         @arg @ref EXTI_LINE_6    
  *         @arg @ref EXTI_LINE_7    
  *         @arg @ref EXTI_LINE_16   
  *         @arg @ref EXTI_LINE_17   
  *         @arg @ref EXTI_LINE_21   
  *         @arg @ref EXTI_LINE_22   
  *         @arg @ref EXTI_LINE_A0_31
  * @note   Please check each device line mapping for EXTI Line availability
  * @retval None
  */
void EXTI_Disable_FallingTrig(uint32_t ExtiLine)
{
  CLEAR_BIT(EXTI->FTSR, ExtiLine);
}


/**
  * @brief  Check if falling edge trigger is enabled for Lines in range 0 to 31
  * @rmtoll EXTI_Get_FallingTrig
  * @param  ExtiLine This parameter can be a combination of the following values:
  *         @arg @ref EXTI_LINE_0    
  *         @arg @ref EXTI_LINE_1    
  *         @arg @ref EXTI_LINE_2    
  *         @arg @ref EXTI_LINE_3    
  *         @arg @ref EXTI_LINE_4    
  *         @arg @ref EXTI_LINE_5    
  *         @arg @ref EXTI_LINE_6    
  *         @arg @ref EXTI_LINE_7    
  *         @arg @ref EXTI_LINE_16   
  *         @arg @ref EXTI_LINE_17   
  *         @arg @ref EXTI_LINE_21   
  *         @arg @ref EXTI_LINE_22   
  *         @arg @ref EXTI_LINE_A0_31
  * @note   Please check each device line mapping for EXTI Line availability
  * @retval State of bit (1 or 0).
  */
uint32_t EXTI_Get_FallingTrig(uint32_t ExtiLine)
{
  return (READ_BIT(EXTI->FTSR, ExtiLine) == (ExtiLine));
}

/**
  * @brief  Generate a software Interrupt Event for Lines in range 0 to 31
  * @note If the interrupt is enabled on this line in the EXTI_IMR, writing a 1 to
  *       this bit when it is at '0' sets the corresponding pending bit in EXTI_PR
  *       resulting in an interrupt request generation.
  *       This bit is cleared by clearing the corresponding bit in the EXTI_PR
  *       register (by writing a 1 into the bit)
  * @rmtoll EXTI_Generate_SWI
  * @param  ExtiLine This parameter can be a combination of the following values:
  *         @arg @ref EXTI_LINE_0    
  *         @arg @ref EXTI_LINE_1    
  *         @arg @ref EXTI_LINE_2    
  *         @arg @ref EXTI_LINE_3    
  *         @arg @ref EXTI_LINE_4    
  *         @arg @ref EXTI_LINE_5    
  *         @arg @ref EXTI_LINE_6    
  *         @arg @ref EXTI_LINE_7    
  *         @arg @ref EXTI_LINE_16   
  *         @arg @ref EXTI_LINE_17   
  *         @arg @ref EXTI_LINE_21   
  *         @arg @ref EXTI_LINE_22   
  *         @arg @ref EXTI_LINE_A0_31
  * @note   Please check each device line mapping for EXTI Line availability
  * @retval None
  */
void EXTI_Generate_SWI(uint32_t ExtiLine)
{
  SET_BIT(EXTI->SWIER, ExtiLine);
}

/**
  * @brief  Check if the ExtLine Flag is set or not for Lines in range 0 to 31
  * @note This bit is set when the selected edge event arrives on the interrupt
  *       line. This bit is cleared by writing a 1 to the bit.
  * @rmtoll EXTI_Get_Flag
  * @param  ExtiLine This parameter can be a combination of the following values:
  *         @arg @ref EXTI_LINE_0    
  *         @arg @ref EXTI_LINE_1    
  *         @arg @ref EXTI_LINE_2    
  *         @arg @ref EXTI_LINE_3    
  *         @arg @ref EXTI_LINE_4    
  *         @arg @ref EXTI_LINE_5    
  *         @arg @ref EXTI_LINE_6    
  *         @arg @ref EXTI_LINE_7    
  *         @arg @ref EXTI_LINE_16   
  *         @arg @ref EXTI_LINE_17   
  *         @arg @ref EXTI_LINE_21   
  *         @arg @ref EXTI_LINE_22   
  *         @arg @ref EXTI_LINE_A0_31
  * @note   Please check each device line mapping for EXTI Line availability
  * @retval State of bit (1 or 0).
  */
uint32_t EXTI_Get_Flag(uint32_t ExtiLine)
{
  return (READ_BIT(EXTI->PR, ExtiLine) == (ExtiLine));
}


/**
  * @brief  Read ExtLine Combination Flag for Lines in range 0 to 31
  * @note This bit is set when the selected edge event arrives on the interrupt
  *       line. This bit is cleared by writing a 1 to the bit.
  * @rmtoll EXTI_Read_Flag
  * @param  ExtiLine This parameter can be a combination of the following values:
  *         @arg @ref EXTI_LINE_0    
  *         @arg @ref EXTI_LINE_1    
  *         @arg @ref EXTI_LINE_2    
  *         @arg @ref EXTI_LINE_3    
  *         @arg @ref EXTI_LINE_4    
  *         @arg @ref EXTI_LINE_5    
  *         @arg @ref EXTI_LINE_6    
  *         @arg @ref EXTI_LINE_7    
  *         @arg @ref EXTI_LINE_16   
  *         @arg @ref EXTI_LINE_17   
  *         @arg @ref EXTI_LINE_21   
  *         @arg @ref EXTI_LINE_22   
  *         @arg @ref EXTI_LINE_A0_31
  * @note   Please check each device line mapping for EXTI Line availability
  * @retval @note This bit is set when the selected edge event arrives on the interrupt
  */
uint32_t EXTI_Read_Flag(uint32_t ExtiLine)
{
  return (uint32_t)(READ_BIT(EXTI->PR, ExtiLine));
}


/**
  * @brief  Clear ExtLine Flags  for Lines in range 0 to 31
  * @note This bit is set when the selected edge event arrives on the interrupt
  *       line. This bit is cleared by writing a 1 to the bit.
  * @rmtoll EXTI_Clear_Flag
  * @param  ExtiLine This parameter can be a combination of the following values:
  *         @arg @ref EXTI_LINE_0    
  *         @arg @ref EXTI_LINE_1    
  *         @arg @ref EXTI_LINE_2    
  *         @arg @ref EXTI_LINE_3    
  *         @arg @ref EXTI_LINE_4    
  *         @arg @ref EXTI_LINE_5    
  *         @arg @ref EXTI_LINE_6    
  *         @arg @ref EXTI_LINE_7    
  *         @arg @ref EXTI_LINE_16   
  *         @arg @ref EXTI_LINE_17   
  *         @arg @ref EXTI_LINE_21   
  *         @arg @ref EXTI_LINE_22   
  *         @arg @ref EXTI_LINE_A0_31
  * @note   Please check each device line mapping for EXTI Line availability
  * @retval None
  */
void EXTI_Clear_Flag(uint32_t ExtiLine)
{
  WRITE_REG(EXTI->PR, ExtiLine);
}

