/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/

/*
*********************************************************************************************************
*                                              rx32s12xx
*                                           Library Function
*
*                                   Copyright 2019, RX Tech, Corp.
*                                        All Rights Reserved
*
*
* Project      : rx32s12xx
* File         : rx32s12xx_i2c.c
* By           : RX_SD_Team
* Version      : V1.0.0
* Description  : Only rx32s12xx
*********************************************************************************************************
*/

#define  __rx32s12xx_I2C_C

#include "rx32s12xx.h"
#include "rx32s12xx_i2c.h"


/**
  * @brief  De-initialize the I2C registers to their default reset values.
  * @param  I2Cx I2C Instance.
  * @retval none.
  */
void I2C_DeInit(I2C_TypeDef *I2Cx)
{
    /* Force reset of I2C clock */
    RCC_Enable_APB1RST(RCC_APB1RSTR_I2C1RST);

    /* Release reset of I2C clock */
    RCC_Disable_APB1RST(RCC_APB1RSTR_I2C1RST);
}

/**
  * @brief  Initialize the I2C registers according to the specified parameters in I2C_InitStruct.
  * @param  I2Cx I2C Instance.
  * @param  ClockSpeed This parameter must be a value lower than 400kHz (in Hz).
  * @param  DutyCycle This parameter can be one of the following values:
  *         @arg @ref I2C_DUTYCYCLE_2
  *         @arg @ref I2C_DUTYCYCLE_16_9
  * @param  OwnAddress1 This parameter must be a value between Min_Data=0 and Max_Data=0x3FF.
  * @param  OwnAddrSize This parameter can be one of the following values:
  *         @arg @ref I2C_OWNADDRESS1_7BIT
  *         @arg @ref I2C_OWNADDRESS1_10BIT
  * @param  TypeAcknowledge This parameter can be one of the following values:
  *         @arg @ref I2C_CR1_ACK
  *         @arg @ref I2C_CR1_NACK
  * @retval None.
  */
void I2C_Init(I2C_TypeDef *I2Cx, I2C_InitTypeDef *I2C_InitStruct)
{
  RCC_ClocksTypeDef rcc_clocks;

  /* Disable the selected I2Cx Peripheral */
  I2C_Disable(I2Cx);

  /* Get SYSCLK frequency */
  rcc_clocks.SYSCLK_Frequency = 80000000;  //RCC_Get_SystemClock()

  /* HCLK clock frequency */
  rcc_clocks.HCLK_Frequency   = 80000000;  //RCC_Get_AHBClock()

  /* PCLK1 clock frequency */
  rcc_clocks.PCLK1_Frequency  = 40000000;  //RCC_Get_APB1Clock()

  /* PCLK2 clock frequency */
  rcc_clocks.PCLK2_Frequency  = 80000000;  //RCC_Get_APB2Clock()

  /*---------------------------- I2Cx SCL Clock Speed Configuration ------------*/
  I2C_Set_Speed(I2Cx, rcc_clocks.PCLK1_Frequency, I2C_InitStruct->ClockSpeed, I2C_InitStruct->DutyCycle);

  /*---------------------------- I2Cx OAR1 Configuration -----------------------*/
  I2C_Set_OwnAddress1(I2Cx, I2C_InitStruct->OwnAddress1, I2C_InitStruct->OwnAddrSize);

  /* Enable the selected I2Cx Peripheral */
  I2C_Enable(I2Cx);

  /*---------------------------- I2Cx CR2 Configuration ------------------------*/
  I2C_Set_AcknowledgeNextData(I2Cx, I2C_InitStruct->TypeAcknowledge);
}
	
/**
  * @brief  Enable I2C peripheral (PE = 1).
  * @rmtoll CR1          PE            I2C_Enable
  * @param  I2Cx I2C Instance.
  * @retval None
  */
void I2C_Enable(I2C_TypeDef *I2Cx)
{
  SET_BIT(I2Cx->CR1, I2C_CR1_PE);
}

/**
  * @brief  Disable I2C peripheral (PE = 0).
  * @rmtoll CR1          PE            I2C_Disable
  * @param  I2Cx I2C Instance.
  * @retval None
  */
void I2C_Disable(I2C_TypeDef *I2Cx)
{
  CLEAR_BIT(I2Cx->CR1, I2C_CR1_PE);
}
/**
  * @brief  Check if the I2C peripheral is enabled or disabled.
  * @rmtoll CR1          PE            I2C_Is_Enabled
  * @param  I2Cx I2C Instance.
  * @retval State of bit (1 or 0).
  */
uint32_t I2C_Is_Enabled(I2C_TypeDef *I2Cx)
{
  return (READ_BIT(I2Cx->CR1, I2C_CR1_PE) == (I2C_CR1_PE));
}

/**
  * @brief  Enable SMBus Packet Error Calculation (PEC).
  * @note   Macro @ref IS_SMBUS_ALL_INSTANCE(I2Cx) can be used to check whether or not
  *         SMBus feature is supported by the I2Cx Instance.
  * @rmtoll CR1          ENPEC         I2C_Enable_SMBusPEC
  * @param  I2Cx I2C Instance.
  * @retval None
  */
void I2C_Enable_SMBusPEC(I2C_TypeDef *I2Cx)
{
  SET_BIT(I2Cx->CR1, I2C_CR1_ENPEC);
}

/**
  * @brief  Disable SMBus Packet Error Calculation (PEC).
  * @note   Macro @ref IS_SMBUS_ALL_INSTANCE(I2Cx) can be used to check whether or not
  *         SMBus feature is supported by the I2Cx Instance.
  * @rmtoll CR1          ENPEC         I2C_Disable_SMBusPEC
  * @param  I2Cx I2C Instance.
  * @retval None
  */
void I2C_Disable_SMBusPEC(I2C_TypeDef *I2Cx)
{
  CLEAR_BIT(I2Cx->CR1, I2C_CR1_ENPEC);
}

/**
  * @brief  Check if SMBus Packet Error Calculation (PEC) is enabled or disabled.
  * @note   Macro @ref IS_SMBUS_ALL_INSTANCE(I2Cx) can be used to check whether or not
  *         SMBus feature is supported by the I2Cx Instance.
  * @rmtoll CR1          ENPEC         I2C_Get_SMBusPECEnabled
  * @param  I2Cx I2C Instance.
  * @retval State of bit (1 or 0).
  */
uint32_t I2C_Get_SMBusPECEnabled(I2C_TypeDef *I2Cx)
{
  return (READ_BIT(I2Cx->CR1, I2C_CR1_ENPEC) == (I2C_CR1_ENPEC));
}

/**
  * @brief  Enable General Call.
  * @note   When enabled the Address 0x00 is ACKed.
  * @rmtoll CR1          ENGC          I2C_Enable_GeneralCall
  * @param  I2Cx I2C Instance.
  * @retval None
  */
void I2C_Enable_GeneralCall(I2C_TypeDef *I2Cx)
{
  SET_BIT(I2Cx->CR1, I2C_CR1_ENGC);
}

/**
  * @brief  Disable General Call.
  * @note   When disabled the Address 0x00 is NACKed.
  * @rmtoll CR1          ENGC          I2C_Disable_GeneralCall
  * @param  I2Cx I2C Instance.
  * @retval None
  */
void I2C_Disable_GeneralCall(I2C_TypeDef *I2Cx)
{
  CLEAR_BIT(I2Cx->CR1, I2C_CR1_ENGC);
}

/**
  * @brief  Check if General Call is enabled or disabled.
  * @rmtoll CR1          ENGC          I2C_Get_GeneralCall
  * @param  I2Cx I2C Instance.
  * @retval State of bit (1 or 0).
  */
uint32_t I2C_Get_GeneralCall(I2C_TypeDef *I2Cx)
{
  return (READ_BIT(I2Cx->CR1, I2C_CR1_ENGC) == (I2C_CR1_ENGC));
}

/**
  * @brief  Enable Clock stretching.
  * @note   This bit can only be programmed when the I2C is disabled (PE = 0).
  * @rmtoll CR1          NOSTRETCH     I2C_Enable_ClockStretching
  * @param  I2Cx I2C Instance.
  * @retval None
  */
void I2C_Enable_ClockStretching(I2C_TypeDef *I2Cx)
{
  CLEAR_BIT(I2Cx->CR1, I2C_CR1_NOSTRETCH);
}

/**
  * @brief  Disable Clock stretching.
  * @note   This bit can only be programmed when the I2C is disabled (PE = 0).
  * @rmtoll CR1          NOSTRETCH     I2C_Disable_ClockStretching
  * @param  I2Cx I2C Instance.
  * @retval None
  */
void I2C_Disable_ClockStretching(I2C_TypeDef *I2Cx)
{
  SET_BIT(I2Cx->CR1, I2C_CR1_NOSTRETCH);
}

/**
  * @brief  Check if Clock stretching is enabled or disabled.
  * @rmtoll CR1          NOSTRETCH     I2C_Get_ClockStretching
  * @param  I2Cx I2C Instance.
  * @retval State of bit (1 or 0).
  */
uint32_t I2C_Get_ClockStretching(I2C_TypeDef *I2Cx)
{
  return (READ_BIT(I2Cx->CR1, I2C_CR1_NOSTRETCH) != (I2C_CR1_NOSTRETCH));
}

/**
  * @brief  Generate a START or RESTART condition
  * @note   The START bit can be set even if bus is BUSY or I2C is in slave mode.
  *         This action has no effect when RELOAD is set.
  * @rmtoll CR1          START         I2C_Generate_StartCondition
  * @param  I2Cx I2C Instance.
  * @retval None
  */
void I2C_Generate_StartCondition(I2C_TypeDef *I2Cx)
{
  SET_BIT(I2Cx->CR1, I2C_CR1_START);
}
/**
  * @brief  Generate a STOP condition after the current byte transfer (master mode).
  * @rmtoll CR1          STOP          I2C_Generate_StopCondition
  * @param  I2Cx I2C Instance.
  * @retval None
  */
void I2C_Generate_StopCondition(I2C_TypeDef *I2Cx)
{
  SET_BIT(I2Cx->CR1, I2C_CR1_STOP);
}

/**
  * @brief  Prepare the generation of a ACKnowledge or Non ACKnowledge condition after the address receive match code or next received byte.
  * @note   Usage in Slave or Master mode.
  * @rmtoll CR1          ACK           I2C_Set_AcknowledgeNextData
  * @param  I2Cx I2C Instance.
  * @param  TypeAcknowledge This parameter can be one of the following values:
  *         @arg @ref I2C_CR1_ACK
  *         @arg @ref I2C_CR1_NACK
  * @retval None
  */
void I2C_Set_AcknowledgeNextData(I2C_TypeDef *I2Cx, uint32_t TypeAcknowledge)
{
  MODIFY_REG(I2Cx->CR1, I2C_CR1_ACK, TypeAcknowledge);
}

/**
  * @brief  Enable bit POS (master/host mode).
  * @note   In that case, the ACK bit controls the (N)ACK of the next byte received or the PEC bit indicates that the next byte in shift register is a PEC.
  * @rmtoll CR1          POS           I2C_Enable_POSBit
  * @param  I2Cx I2C Instance.
  * @retval None
  */
void I2C_Enable_POSBit(I2C_TypeDef *I2Cx)
{
  SET_BIT(I2Cx->CR1, I2C_CR1_POS);
}

/**
  * @brief  Disable bit POS (master/host mode).
  * @note   In that case, the ACK bit controls the (N)ACK of the current byte received or the PEC bit indicates that the current byte in shift register is a PEC.
  * @rmtoll CR1          POS           I2C_Disable_POSBit
  * @param  I2Cx I2C Instance.
  * @retval None
  */
void I2C_Disable_POSBit(I2C_TypeDef *I2Cx)
{
  CLEAR_BIT(I2Cx->CR1, I2C_CR1_POS);
}

/**
  * @brief  Check if bit POS  is enabled or disabled.
  * @rmtoll CR1          POS           I2C_Get_POSBit
  * @param  I2Cx I2C Instance.
  * @retval State of bit (1 or 0).
  */
uint32_t I2C_Get_POSBit(I2C_TypeDef *I2Cx)
{
  return (READ_BIT(I2Cx->CR1, I2C_CR1_POS) == (I2C_CR1_POS));
}

/**
  * @brief  Enable transfer or internal comparison of the SMBus Packet Error byte (transmission or reception mode).
  * @note   Macro @ref IS_SMBUS_ALL_INSTANCE(I2Cx) can be used to check whether or not
  *         SMBus feature is supported by the I2Cx Instance.
  * @note   This feature is cleared by hardware when the PEC byte is transferred or compared,
  *         or by a START or STOP condition, it is also cleared by software.
  * @rmtoll CR1          PEC           I2C_Enable_SMBusPECCompare
  * @param  I2Cx I2C Instance.
  * @retval None
  */
void I2C_Enable_SMBusPECCompare(I2C_TypeDef *I2Cx)
{
  SET_BIT(I2Cx->CR1, I2C_CR1_PEC);
}

/**
  * @brief  Disable transfer or internal comparison of the SMBus Packet Error byte (transmission or reception mode).
  * @note   Macro @ref IS_SMBUS_ALL_INSTANCE(I2Cx) can be used to check whether or not
  *         SMBus feature is supported by the I2Cx Instance.
  * @rmtoll CR1          PEC           I2C_Disable_SMBusPECCompare
  * @param  I2Cx I2C Instance.
  * @retval None
  */
void I2C_Disable_SMBusPECCompare(I2C_TypeDef *I2Cx)
{
  CLEAR_BIT(I2Cx->CR1, I2C_CR1_PEC);
}

/**
  * @brief  Check if the SMBus Packet Error byte transfer or internal comparison is requested or not.
  * @note   Macro @ref IS_SMBUS_ALL_INSTANCE(I2Cx) can be used to check whether or not
  *         SMBus feature is supported by the I2Cx Instance.
  * @rmtoll CR1          PEC           I2C_Get_SMBusPECCompare
  * @param  I2Cx I2C Instance.
  * @retval State of bit (1 or 0).
  */
uint32_t I2C_Get_SMBusPECCompare(I2C_TypeDef *I2Cx)
{
  return (READ_BIT(I2Cx->CR1, I2C_CR1_PEC) == (I2C_CR1_PEC));
}

/**
  * @brief  Enable Reset of I2C peripheral.
  * @rmtoll CR1          SWRST         I2C_Enable_Reset
  * @param  I2Cx I2C Instance.
  * @retval None
  */
void I2C_Enable_Reset(I2C_TypeDef *I2Cx)
{
  SET_BIT(I2Cx->CR1, I2C_CR1_SWRST);
}

/**
  * @brief  Disable Reset of I2C peripheral.
  * @rmtoll CR1          SWRST         I2C_Disable_Reset
  * @param  I2Cx I2C Instance.
  * @retval None
  */
void I2C_Disable_Reset(I2C_TypeDef *I2Cx)
{
  CLEAR_BIT(I2Cx->CR1, I2C_CR1_SWRST);
}

/**
  * @brief  Check if the I2C peripheral is under reset state or not.
  * @rmtoll CR1          SWRST         I2C_Get_Reset
  * @param  I2Cx I2C Instance.
  * @retval State of bit (1 or 0).
  */
uint32_t I2C_Get_Reset(I2C_TypeDef *I2Cx)
{
  return (READ_BIT(I2Cx->CR1, I2C_CR1_SWRST) == (I2C_CR1_SWRST));
}


/** @defgroup I2C_Congig_CR2
  * @{
  */
/**
  * @brief  Configure the Peripheral clock frequency.
  * @rmtoll CR2          FREQ          I2C_Set_PeriphClockFREQ
  * @param  I2Cx I2C Instance.
  * @param  PeriphClock Peripheral Clock (in Hz)
  * @retval None
  */
void I2C_Set_PeriphClockFREQ(I2C_TypeDef *I2Cx, uint32_t PeriphClock)
{
  MODIFY_REG(I2Cx->CR2, I2C_CR2_FREQ, (uint32_t)(PeriphClock/1000000U));
}

/**
  * @brief  Get the Peripheral clock frequency.
  * @rmtoll CR2          FREQ          I2C_Get_PeriphClockFREQ
  * @param  I2Cx I2C Instance.
  * @retval Value of Peripheral Clock (in Hz)
  */
uint32_t I2C_Get_PeriphClockFREQ(I2C_TypeDef *I2Cx)
{
  return (uint32_t)(1000000U*(READ_BIT(I2Cx->CR2, I2C_CR2_FREQ)));
}


/**
***********************************************************************************
*@Function I2C_Enable_IT
*@brief    Enable I2C Iterrupt.
*@param  I2Cx I2C Instance
*@param1   IT:
*          @arg  I2C_CR2_ITERREN
*          @arg  I2C_CR2_ITEVTEN
*          @arg  I2C_CR2_ITBUFEN
*          @arg  I2C_CR2_ITEVTEN | I2C_CR2_ITBUFEN    (TXE interrupt)
*@retval None.
***********************************************************************************
  */
void I2C_Enable_IT(I2C_TypeDef *I2Cx, uint32_t IT)
{
	SET_BIT( I2Cx->CR2, IT );
}

/**
***********************************************************************************
*@Function I2C_Disable_IT
*@brief    Disable I2C Iterrupt.
*@param  I2Cx I2C Instance
*@param1   IT:
*          @arg  I2C_CR2_ITERREN
*          @arg  I2C_CR2_ITEVTEN
*          @arg  I2C_CR2_ITBUFEN
*          @arg  I2C_CR2_ITEVTEN | I2C_CR2_ITBUFEN    (TXE interrupt)
*@retval None.
***********************************************************************************
  */
void I2C_Disable_IT(I2C_TypeDef *I2Cx, uint32_t IT)
{
	CLEAR_BIT( I2Cx->CR2, IT );
}


/**
***********************************************************************************
*@Function I2C_Get_IT
*@brief    Get I2C Iterrupt.
*@param  I2Cx I2C Instance
*@param1   IT:
*          @arg  I2C_CR2_ITERREN
*          @arg  I2C_CR2_ITEVTEN
*          @arg  I2C_CR2_ITBUFEN
*          @arg  I2C_CR2_ITEVTEN | I2C_CR2_ITBUFEN    (TXE interrupt)
*@retval State of bit (1 or 0).
***********************************************************************************
  */
uint32_t I2C_Get_IT(I2C_TypeDef *I2Cx, uint32_t IT)
{
	return (READ_BIT(I2Cx->CR2, IT) == (IT));
}

/**
  * @brief  Enable DMA transmission requests.
  * @rmtoll CR2          DMAEN         I2C_Enable_DMAReq
  * @param  I2Cx I2C Instance.
  * @retval None
  */
void I2C_Enable_DMAReq(I2C_TypeDef *I2Cx)
{
  SET_BIT(I2Cx->CR2, I2C_CR2_DMAEN);
}

/**
  * @brief  Disable DMA transmission requests.
  * @rmtoll CR2          DMAEN         I2C_Disable_DMAReq
  * @param  I2Cx I2C Instance.
  * @retval None
  */
void I2C_Disable_DMAReq(I2C_TypeDef *I2Cx)
{
  CLEAR_BIT(I2Cx->CR2, I2C_CR2_DMAEN);
}

/**
  * @brief  Check if DMA transmission requests are enabled or disabled.
  * @rmtoll CR2          DMAEN         I2C_Get_DMAReq
  * @param  I2Cx I2C Instance.
  * @retval State of bit (1 or 0).
  */
uint32_t I2C_Get_DMAReq(I2C_TypeDef *I2Cx)
{
  return (READ_BIT(I2Cx->CR2, I2C_CR2_DMAEN) == (I2C_CR2_DMAEN));
}

/**
  * @brief  Enable DMA last transfer.
  * @note   This action mean that next DMA EOT is the last transfer.
  * @rmtoll CR2          LAST          I2C_Enable_LastDMA
  * @param  I2Cx I2C Instance.
  * @retval None
  */
void I2C_Enable_LastDMA(I2C_TypeDef *I2Cx)
{
  SET_BIT(I2Cx->CR2, I2C_CR2_LAST);
}

/**
  * @brief  Disable DMA last transfer.
  * @note   This action mean that next DMA EOT is not the last transfer.
  * @rmtoll CR2          LAST          I2C_Disable_LastDMA
  * @param  I2Cx I2C Instance.
  * @retval None
  */
void I2C_Disable_LastDMA(I2C_TypeDef *I2Cx)
{
  CLEAR_BIT(I2Cx->CR2, I2C_CR2_LAST);
}

/**
  * @brief  Check if DMA last transfer is enabled or disabled.
  * @rmtoll CR2          LAST          I2C_Get_LastDMA
  * @param  I2Cx I2C Instance.
  * @retval State of bit (1 or 0).
  */
uint32_t I2C_Get_LastDMA(I2C_TypeDef *I2Cx)
{
  return (READ_BIT(I2Cx->CR2, I2C_CR2_LAST) == (I2C_CR2_LAST));
}

/**
  * @brief  Set the Own Address1.
  * @rmtoll OAR1         ADD0          I2C_Set_OwnAddress1\n
  *         OAR1         ADD1_7        I2C_Set_OwnAddress1\n
  *         OAR1         ADD8_9        I2C_Set_OwnAddress1\n
  *         OAR1         ADDMODE       I2C_Set_OwnAddress1
  * @param  I2Cx I2C Instance.
  * @param  OwnAddress1 This parameter must be a value between Min_Data=0 and Max_Data=0x3FF.
  * @param  OwnAddrSize This parameter can be one of the following values:
  *         @arg @ref I2C_OWNADDRESS1_7BIT
  *         @arg @ref I2C_OWNADDRESS1_10BIT
  * @retval None
  */
void I2C_Set_OwnAddress1(I2C_TypeDef *I2Cx, uint32_t OwnAddress1, uint32_t OwnAddrSize)
{
  MODIFY_REG(I2Cx->OAR1, I2C_OAR1_ADD0 | I2C_OAR1_ADD1_7 | I2C_OAR1_ADD8_9 | I2C_OAR1_ADDMODE, OwnAddress1 | OwnAddrSize);
}

/**
  * @brief  Enable acknowledge on Own Address2 match address.
  * @rmtoll OAR2         ENDUAL        I2C_Enable_OwnAddress2
  * @param  I2Cx I2C Instance.
  * @retval None
  */
void I2C_Enable_OwnAddress2(I2C_TypeDef *I2Cx)
{
  SET_BIT(I2Cx->OAR2, I2C_OAR2_ENDUAL);
}

/**
  * @brief  Disable  acknowledge on Own Address2 match address.
  * @rmtoll OAR2         ENDUAL        I2C_Disable_OwnAddress2
  * @param  I2Cx I2C Instance.
  * @retval None
  */
void I2C_Disable_OwnAddress2(I2C_TypeDef *I2Cx)
{
  CLEAR_BIT(I2Cx->OAR2, I2C_OAR2_ENDUAL);
}

/**
  * @brief  Check if Own Address1 acknowledge is enabled or disabled.
  * @rmtoll OAR2         ENDUAL        I2C_Get_OwnAddress2
  * @param  I2Cx I2C Instance.
  * @retval State of bit (1 or 0).
  */
uint32_t I2C_Get_OwnAddress2(I2C_TypeDef *I2Cx)
{
  return (READ_BIT(I2Cx->OAR2, I2C_OAR2_ENDUAL) == (I2C_OAR2_ENDUAL));
}

/**
  * @brief  Set the 7bits Own Address2.
  * @note   This action has no effect if own address2 is enabled.
  * @rmtoll OAR2         ADD2          I2C_Set_OwnAddress2
  * @param  I2Cx I2C Instance.
  * @param  OwnAddress2 This parameter must be a value between Min_Data=0 and Max_Data=0x7F.
  * @retval None
  */
void I2C_Set_OwnAddress2(I2C_TypeDef *I2Cx, uint32_t OwnAddress2)
{
  MODIFY_REG(I2Cx->OAR2, I2C_OAR2_ADD2, OwnAddress2);
}

/**
  * @brief  Get the data register address used for DMA transfer.
  * @rmtoll DR           DR            I2C_Get_DMARegAddr
  * @param  I2Cx I2C Instance.
  * @retval Address of data register
  */
uint32_t I2C_Get_DMARegAddr(I2C_TypeDef *I2Cx)
{
  return (uint32_t) & (I2Cx->DR);
}

/**
  * @brief  Read Receive Data register.
  * @rmtoll DR           DR            I2C_Receive_Data8
  * @param  I2Cx I2C Instance.
  * @retval Value between Min_Data=0x0 and Max_Data=0xFF
  */
uint8_t I2C_Receive_Data8(I2C_TypeDef *I2Cx)
{
  return (uint8_t)(READ_BIT(I2Cx->DR, I2C_DR_DR));
}

/**
  * @brief  Write in Transmit Data Register .
  * @rmtoll DR           DR            I2C_Transmit_Data8
  * @param  I2Cx I2C Instance.
  * @param  Data Value between Min_Data=0x0 and Max_Data=0xFF
  * @retval None
  */
void I2C_Transmit_Data8(I2C_TypeDef *I2Cx, uint8_t Data)
{
  MODIFY_REG(I2Cx->DR, I2C_DR_DR, Data);
}

/**
  * @brief  Indicate the status of SR1.
  * @rmtoll SR1                     I2C_Get_Flag
  * @param  I2Cx I2C Instance.
  *         @arg @ref  I2C_FLAG_SB     
  *         @arg @ref  I2C_FLAG_ADDR   
  *         @arg @ref  I2C_FLAG_BTF    
  *         @arg @ref  I2C_FLAG_ADD10  
  *         @arg @ref  I2C_FLAG_STOPF  
  *         @arg @ref  I2C_FLAG_RXNE   
  *         @arg @ref  I2C_FLAG_TXE     
  *         @arg @ref  I2C_FLAG_BERR   
  *         @arg @ref  I2C_FLAG_ARLO    
  *         @arg @ref  I2C_FLAG_AF    
  *         @arg @ref  I2C_FLAG_OVR     
  *         @arg @ref  I2C_FLAG_PECERR 
  *         @arg @ref  I2C_FLAG_TIMEOUT 
  *         @arg @ref  I2C_FLAG_MSL     
  *         @arg @ref  I2C_FLAG_BUSY   
  *         @arg @ref  I2C_FLAG_GENCALL
  *         @arg @ref  I2C_FLAG_DUALF     
  * @retval State of bit (1 or 0).
  */
uint32_t I2C_Get_Flag(I2C_TypeDef *I2Cx,uint32_t FLAG)
{
	if((FLAG&0x10000000)==0x10000000)
	{
		return (READ_BIT(I2Cx->SR1, (FLAG&0xFFFFFFF)) == (FLAG&0xFFFFFFF));
	}
	else if((FLAG&0x20000000)==0x20000000)
	{
		return (READ_BIT(I2Cx->SR2, (FLAG&0x1FFFFFFF)) == (FLAG&0x1FFFFFFF));
	}
	return 0;
}

/**
  * @brief  Clear the status of SR1.
  * @rmtoll SR1             I2C_Clear_Flag
  * @param  I2Cx I2C Instance.  
  *         @arg @ref  I2C_FLAG_ADDR
  *         @arg @ref  I2C_FLAG_STOPF
  *         @arg @ref  I2C_FLAG_BERR
  *         @arg @ref  I2C_FLAG_ARLO 
  *         @arg @ref  I2C_FLAG_AF
  *         @arg @ref  I2C_FLAG_OVR 
  *         @arg @ref  I2C_FLAG_PECERR
  *         @arg @ref  I2C_FLAG_TIMEOUT 
  * @retval None.
  */
void I2C_Clear_Flag(I2C_TypeDef *I2Cx,uint32_t FLAG)
{
	__IO uint32_t tmpreg;
	if( (FLAG&I2C_FLAG_ADDR) == I2C_FLAG_ADDR )
	{
		tmpreg = I2Cx->SR1;
	    (void) tmpreg;
	    tmpreg = I2Cx->SR2;
	    (void) tmpreg;
	}
	else if( (FLAG&I2C_FLAG_STOPF) == I2C_FLAG_STOPF)
	{
		tmpreg = I2Cx->SR1;
		(void) tmpreg;
		SET_BIT(I2Cx->CR1, I2C_CR1_PE);
	}
	else if((FLAG&0x10000000)==0x10000000)
	{
		CLEAR_BIT(I2Cx->SR1, (FLAG&0xFFFFFFF));
	}
}


/**
  * @brief  Indicate the value of transfer direction.
  * @note   RESET: Bus is in read transfer (peripheral point of view).
  *         SET: Bus is in write transfer (peripheral point of view).
  * @rmtoll SR2          TRA           I2C_Get_TransferDirection
  * @param  I2Cx I2C Instance.
  * @retval Returned value can be one of the following values:
  *         @arg @ref I2C_DIRECTION_WRITE
  *         @arg @ref I2C_DIRECTION_READ
  */
uint32_t I2C_Get_TransferDirection(I2C_TypeDef *I2Cx)
{
  return (uint32_t)(READ_BIT(I2Cx->SR2, I2C_SR2_TRA));
}

/**
  * @brief  Get the SMBus Packet Error byte calculated.
  * @note   Macro @ref IS_SMBUS_ALL_INSTANCE(I2Cx) can be used to check whether or not
  *         SMBus feature is supported by the I2Cx Instance.
  * @rmtoll SR2          PEC           I2C_Get_SMBusPEC
  * @param  I2Cx I2C Instance.
  * @retval Value between Min_Data=0x00 and Max_Data=0xFF
  */
uint32_t I2C_Get_SMBusPEC(I2C_TypeDef *I2Cx)
{
  return (uint32_t)(READ_BIT(I2Cx->SR2, I2C_SR2_PEC) >> 8U);
}

/**
  * @brief  Configure the SCL high and low period.
  * @note   This bit can only be programmed when the I2C is disabled (PE = 0).
  * @rmtoll CCR          CCR           I2C_Set_ClockPeriod
  * @param  I2Cx I2C Instance.
  * @param  ClockPeriod This parameter must be a value between Min_Data=0x004 and Max_Data=0xFFF, except in FAST DUTY mode where Min_Data=0x001.
  * @retval None
  */
void I2C_Set_ClockPeriod(I2C_TypeDef *I2Cx, uint32_t ClockPeriod)
{
  MODIFY_REG(I2Cx->CCR, I2C_CCR_CCR, ClockPeriod);
}

/**
  * @brief  Get the SCL high and low period.
  * @rmtoll CCR          CCR           I2C_Get_ClockPeriod
  * @param  I2Cx I2C Instance.
  * @retval Value between Min_Data=0x004 and Max_Data=0xFFF, except in FAST DUTY mode where Min_Data=0x001.
  */
uint32_t I2C_Get_ClockPeriod(I2C_TypeDef *I2Cx)
{
  return (uint32_t)(READ_BIT(I2Cx->CCR, I2C_CCR_CCR));
}


/**
  * @brief  Configure the Duty cycle (Fast mode only).
  * @rmtoll CCR          DUTY          I2C_Set_DutyCycle
  * @param  I2Cx I2C Instance.
  * @param  DutyCycle This parameter can be one of the following values:
  *         @arg @ref I2C_DUTYCYCLE_2
  *         @arg @ref I2C_DUTYCYCLE_16_9
  * @retval None
  */
void I2C_Set_DutyCycle(I2C_TypeDef *I2Cx, uint32_t DutyCycle)
{
  MODIFY_REG(I2Cx->CCR, I2C_CCR_DUTY, DutyCycle);
}

/**
  * @brief  Get the Duty cycle (Fast mode only).
  * @rmtoll CCR          DUTY          I2C_Get_DutyCycle
  * @param  I2Cx I2C Instance.
  * @retval Returned value can be one of the following values:
  *         @arg @ref I2C_DUTYCYCLE_2
  *         @arg @ref I2C_DUTYCYCLE_16_9
  */
uint32_t I2C_Get_DutyCycle(I2C_TypeDef *I2Cx)
{
  return (uint32_t)(READ_BIT(I2Cx->CCR, I2C_CCR_DUTY));
}

/**
  * @brief  Configure the I2C master clock speed mode.
  * @rmtoll CCR          FS            I2C_Set_ClockSpeedMode
  * @param  I2Cx I2C Instance.
  * @param  ClockSpeedMode This parameter can be one of the following values:
  *         @arg @ref I2C_STANDARD_MODE
  *         @arg @ref I2C_FAST_MODE
  * @retval None
  */
void I2C_Set_ClockSpeedMode(I2C_TypeDef *I2Cx, uint32_t ClockSpeedMode)
{
  MODIFY_REG(I2Cx->CCR, I2C_CCR_FS, ClockSpeedMode);
}

/**
  * @brief  Get the the I2C master speed mode.
  * @rmtoll CCR          FS            I2C_Get_ClockSpeedMode
  * @param  I2Cx I2C Instance.
  * @retval Returned value can be one of the following values:
  *         @arg @ref I2C_STANDARD_MODE
  *         @arg @ref I2C_FAST_MODE
  */
uint32_t I2C_Get_ClockSpeedMode(I2C_TypeDef *I2Cx)
{
  return (uint32_t)(READ_BIT(I2Cx->CCR, I2C_CCR_FS));
}

/**
  * @brief  Configure the SCL speed.
  * @note   This bit can only be programmed when the I2C is disabled (PE = 0).
  * @rmtoll CR2          FREQ          I2C_Set_Speed\n
  *         TRISE        TRISE         I2C_Set_Speed\n
  *         CCR          FS            I2C_Set_Speed\n
  *         CCR          DUTY          I2C_Set_Speed\n
  *         CCR          CCR           I2C_Set_Speed
  * @param  I2Cx I2C Instance.
  * @param  PeriphClock Peripheral Clock (in Hz)
  * @param  ClockSpeed This parameter must be a value lower than 400kHz (in Hz).
  * @param  DutyCycle This parameter can be one of the following values:
  *         @arg @ref I2C_DUTYCYCLE_2
  *         @arg @ref I2C_DUTYCYCLE_16_9
  * @retval None
  */
void I2C_Set_Speed(I2C_TypeDef *I2Cx, uint32_t PeriphClock, uint32_t ClockSpeed,
                                        uint32_t DutyCycle)
{
  register uint32_t freqrange = 0x0U;
  register uint32_t clockconfig = 0x0U;

  /* Compute frequency range */
  freqrange = (uint32_t)((PeriphClock)/1000000U);

  /* Configure I2Cx: Frequency range register */
  MODIFY_REG(I2Cx->CR2, I2C_CR2_FREQ, freqrange);

  /* Configure I2Cx: Rise Time register */
  MODIFY_REG(I2Cx->TRISE, I2C_TRISE_TRISE, __I2C_RISE_TIME(freqrange, ClockSpeed));

  /* Configure Speed mode, Duty Cycle and Clock control register value */
  if (ClockSpeed > I2C_MAX_SPEED_STANDARD)
  {
    /* Set Speed mode at fast and duty cycle for Clock Speed request in fast clock range */
    clockconfig = I2C_FAST_MODE                                          | \
                  __I2C_SPEED_FAST_TO_CCR(PeriphClock, ClockSpeed, DutyCycle)        | \
                  DutyCycle;
  }
  else
  {
    /* Set Speed mode at standard for Clock Speed request in standard clock range */
    clockconfig = I2C_STANDARD_MODE                                      | \
                  __I2C_SPEED_STANDARD_TO_CCR(PeriphClock, ClockSpeed);
  }

  /* Configure I2Cx: Clock control register */
  MODIFY_REG(I2Cx->CCR, (I2C_CCR_FS | I2C_CCR_DUTY | I2C_CCR_CCR), clockconfig);
}

/**
  * @brief  Configure the SCL, SDA rising time.
  * @note   This bit can only be programmed when the I2C is disabled (PE = 0).
  * @rmtoll TRISE        TRISE         I2C_Set_RiseTime
  * @param  I2Cx I2C Instance.
  * @param  RiseTime This parameter must be a value between Min_Data=0x02 and Max_Data=0x3F.
  * @retval None
  */
void I2C_Set_RiseTime(I2C_TypeDef *I2Cx, uint32_t RiseTime)
{
  MODIFY_REG(I2Cx->TRISE, I2C_TRISE_TRISE, RiseTime);
}

/**
  * @brief  Get the SCL, SDA rising time.
  * @rmtoll TRISE        TRISE         I2C_Get_RiseTime
  * @param  I2Cx I2C Instance.
  * @retval Value between Min_Data=0x02 and Max_Data=0x3F
  */
uint32_t I2C_Get_RiseTime(I2C_TypeDef *I2Cx)
{
  return (uint32_t)(READ_BIT(I2Cx->TRISE, I2C_TRISE_TRISE));
}

