/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/


/*
*********************************************************************************************************
*                                              rx32s12xx
*                                           Library Function
*
*                                   Copyright 2021, RX Tech, Corp.
*                                        All Rights Reserved
*
*
* Project      : rx32s12xx
* File         : rx32s12xx_rtc.c
* By           : RX_SD_Team
* Version      : V1.0.0
* Description  : Only rx32s12xx
*********************************************************************************************************
*/

#include "rx32s12xx_rtc.h"


/**
  * @brief  De-Initializes the RTC registers to their default reset values.
  * @note   This function doesn't reset the RTC Clock source and RTC Backup Data
  *         registers.
  * @param  RTCx RTC Instance
  * @retval None.
  */
void RTC_DeInit(RTC_TypeDef *RTCx)
{
  /* Disable the write protection for RTC registers */
  RTC_Disable_WriteProtection(RTCx);

  /* Set Initialization mode */
  if (RTC_Enter_InitMode(RTCx) != 1)
  {
    RTCx->CNTL = 0x0000;
    RTCx->CNTH = 0x0000;
    RTCx->PRLH = 0x0000;
    RTCx->PRLL = 0x8000;
    RTCx->CRH = 0x0000;
    RTCx->CRL = 0x0020;

    /* Exit Initialization Mode */
    if (RTC_Exit_InitMode(RTCx) != 1)
    {
      /* Wait till the RTC RSF flag is set */
      RTC_Wait_Synchro();
  
      /* Clear RSF Flag */
      RTC_Clear_Flag(RTC_FLAG_RSF);
      
      /* Enable the write protection for RTC registers */
      RTC_Enable_WriteProtection(RTCx);
    }
  }
  else
  {
    /* Enable the write protection for RTC registers */
    RTC_Enable_WriteProtection(RTCx);
  }
}

/**
  * @brief  Set each @ref RTC_InitTypeDef field to default value.
  * @param  RTC_InitStruct pointer to a @ref RTC_InitTypeDef structure which will be initialized.
  * @retval None
  */
void RTC_StructInit(RTC_InitTypeDef *RTC_InitStruct)
{
  /* Set RTC_InitStruct fields to default values */
  RTC_InitStruct->AsynchPrescaler = RTC_ASYNCH_PRESC_DEFAULT;
//  RTC_InitStruct->OutPutSource    = BKP_RTCOutputSource_None;
}

/**
  * @brief  Set each @ref RTC_TimeTypeDef field to default value (Time = 00h:00min:00sec).
  * @param  RTC_TimeStruct pointer to a @ref RTC_TimeTypeDef structure which will be initialized.
  * @retval None
  */
void RTC_TIME_StructInit(RTC_TimeTypeDef *RTC_TimeStruct)
{
  /* Time = 00h:00min:00sec */
  RTC_TimeStruct->Hours      = 0U;
  RTC_TimeStruct->Minutes    = 0U;
  RTC_TimeStruct->Seconds    = 0U;
}

/**
  * @brief  Set each @ref RTC_AlarmTypeDef of ALARM field to default value (Time = 00h:00mn:00sec /
  *         Day = 1st day of the month/Mask = all fields are masked).
  * @param  RTC_AlarmStruct pointer to a @ref RTC_AlarmTypeDef structure which will be initialized.
  * @retval None
  */
void RTC_ALARM_StructInit(RTC_AlarmTypeDef *RTC_AlarmStruct)
{
  /* Alarm Time Settings : Time = 00h:00mn:00sec */
  RTC_AlarmStruct->AlarmTime.Hours      = 0U;
  RTC_AlarmStruct->AlarmTime.Minutes    = 0U;
  RTC_AlarmStruct->AlarmTime.Seconds    = 0U;
}

/**
  * @brief  Initializes the RTC registers according to the specified parameters
  *         in RTC_InitStruct.
  * @param  RTCx RTC Instance
  * @param  AsynchPrescaler Value between Min_Data = 0 and Max_Data = 0xFFFFF
  * @param  OutputSource This parameter can be one of the following values:
  *         @arg @ref BKP_RTCOutputSource_None      
  *         @arg @ref BKP_RTCOutputSource_CalibClock
  *         @arg @ref BKP_RTCOutputSource_Alarm     
  *         @arg @ref BKP_RTCOutputSource_Second  
  * @note   The RTC Prescaler register is write protected and can be written in
  *         initialization mode only.
  * @note   the user should call RTC_StructInit()  or the structure of Prescaler
  *         need to be initialized  before RTC init()
  * @retval None.
  */
void RTC_Initial(RTC_TypeDef *RTCx, RTC_InitTypeDef *RTC_InitStruct)
{
  /* Waiting for synchro */
  RTC_Wait_Synchro();
  
  /* Set Initialization mode */
  if (RTC_Enter_InitMode(RTCx) != 1)
  {
    /* Clear Flag Bits */
    RTC_Clear_Flag(RTC_FLAG_ALR);
    RTC_Clear_Flag(RTC_FLAG_OW);
    RTC_Clear_Flag(RTC_FLAG_SEC);

//    if (RTC_InitStruct->OutPutSource != BKP_RTCOutputSource_None)
//    {
//      /* Disable the selected Tamper Pin */
//      BKP_Disable_TamperPin();
//    }
//    /* Set the signal which will be routed to RTC Tamper Pin */
//    BKP_Set_RTCOutput(BKP, RTC_InitStruct->OutPutSource);

    /* Configure Synchronous and Asynchronous prescaler factor */
    RTC_Set_AsynchPrescaler(RTCx, RTC_InitStruct->AsynchPrescaler);

    /* Exit Initialization Mode */
    RTC_Exit_InitMode(RTCx);
  }
}

/**
  * @brief  Set the RTC current time.
  * @param  RTCx RTC Instance
  * @param  RTC_Format This parameter can be one of the following values:
  *         @arg @ref RTC_FORMAT_BIN
  *         @arg @ref RTC_FORMAT_BCD
  * @param  Hours This parameter must be a number between Min_Data = 0 and Max_Data = 23 
  * @param  Minutes This parameter must be a number between Min_Data = 0 and Max_Data = 59
  * @param  Seconds This parameter must be a number between Min_Data = 0 and Max_Data = 59 
  * @param  RTC_TimeStruct pointer to a RTC_TimeTypeDef structure that contains
  *                        the time configuration information for the RTC.
  * @note  The user should call RTC_TIME_StructInit() or the structure
  *        of time need to be initialized  before time init()
  * @retval None.
  */
void RTC_TIME_Init(RTC_TypeDef *RTCx, RTC_TimeTypeDef *RTC_TimeStruct)
{
  uint32_t counter_time = 0U;

  /* Enter Initialization mode */
  if (RTC_Enter_InitMode(RTCx) != 1)
  {
    /* Check the input parameters format */
    if (RTC_TimeStruct->Format != RTC_FORMAT_BIN)
    {
      counter_time = (uint32_t)(((uint32_t)RTC_TimeStruct->Hours * 3600U) + \
                                ((uint32_t)RTC_TimeStruct->Minutes * 60U) + \
                                ((uint32_t)RTC_TimeStruct->Seconds));
      RTC_Set_TIME(RTCx, counter_time);
    }
    else
    {
      counter_time = (((uint32_t)(__RTC_CONVERT_BCD2BIN(RTC_TimeStruct->Hours)) * 3600U) + \
                      ((uint32_t)(__RTC_CONVERT_BCD2BIN(RTC_TimeStruct->Minutes)) * 60U) + \
                      ((uint32_t)(__RTC_CONVERT_BCD2BIN(RTC_TimeStruct->Seconds))));
      RTC_Set_TIME(RTCx, counter_time);
    }
  }
  /* Exit Initialization mode */
  RTC_Exit_InitMode(RTCx);
}

/**
  * @brief  Set the RTC Alarm.
  * @param  RTCx RTC Instance
  * @param  RTC_Format This parameter can be one of the following values:
  *         @arg @ref RTC_FORMAT_BIN
  *         @arg @ref RTC_FORMAT_BCD
  * @param  Hours This parameter must be a number between Min_Data = 0 and Max_Data = 23 
  * @param  Minutes This parameter must be a number between Min_Data = 0 and Max_Data = 59
  * @param  Seconds This parameter must be a number between Min_Data = 0 and Max_Data = 59 
  * @param  RTC_AlarmStruct pointer to a @ref RTC_AlarmTypeDef structure that
  *                         contains the alarm configuration parameters.
  * @note   the user should call RTC_ALARM_StructInit()  or the structure
  *         of Alarm need to be initialized  before Alarm init()
  * @retval None.
  */
void RTC_ALARM_Init(RTC_TypeDef *RTCx, RTC_AlarmTypeDef *RTC_AlarmStruct)
{
  uint32_t counter_alarm = 0U;

  /* Enter Initialization mode */
  if (RTC_Enter_InitMode(RTCx) != 1)
  {
    /* Check the input parameters format */
    if (RTC_AlarmStruct->AlarmTime.Format != RTC_FORMAT_BIN)
    {
      counter_alarm = (uint32_t)(((uint32_t)RTC_AlarmStruct->AlarmTime.Hours * 3600U) + \
                                 ((uint32_t)RTC_AlarmStruct->AlarmTime.Minutes * 60U) + \
                                 ((uint32_t)RTC_AlarmStruct->AlarmTime.Seconds));
      RTC_Set_ALARM(RTCx, counter_alarm);
    }
    else
    {
      counter_alarm = (((uint32_t)(__RTC_CONVERT_BCD2BIN(RTC_AlarmStruct->AlarmTime.Hours)) * 3600U) + \
                       ((uint32_t)(__RTC_CONVERT_BCD2BIN(RTC_AlarmStruct->AlarmTime.Minutes)) * 60U) + \
                       ((uint32_t)(__RTC_CONVERT_BCD2BIN(RTC_AlarmStruct->AlarmTime.Seconds))));
      RTC_Set_ALARM(RTCx, counter_alarm);
    }
  }
  /* Exit Initialization mode */
  RTC_Exit_InitMode(RTCx);
}

/**
  * @brief  Enters the RTC Initialization mode.
  * @param  RTCx RTC Instance
  * @retval An ErrorStatus enumeration value:
  *          - SUCCESS: 0
  *          - ERROR: 1
  */
uint32_t RTC_Enter_InitMode(RTC_TypeDef *RTCx)
{
  __IO uint32_t timeout = RTC_INITMODE_TIMEOUT;
  uint32_t status = 0;
  uint32_t tmp = 0U;

  /* Wait till RTC is in INIT state and if Time out is reached exit */
  tmp = RTC_Get_Flag(RTC_FLAG_RTOFF);
  while ((timeout != 0U) && (tmp != 1U))
  {
    if ((SysTick->CTRL & SysTick_CTRL_COUNTFLAG_Msk) == (SysTick_CTRL_COUNTFLAG_Msk))
    {
      timeout --;
    }
    tmp = RTC_Get_Flag(RTC_FLAG_RTOFF);
    if (timeout == 0U)
    {
      status = 1;
    }
  }

  /* Disable the write protection for RTC registers */
  RTC_Disable_WriteProtection(RTCx);

  return status;
}

/**
  * @brief  Exit the RTC Initialization mode.
  * @note   When the initialization sequence is complete, the calendar restarts
  *         counting after 4 RTCCLK cycles.
  * @param  RTCx RTC Instance
  * @retval An ErrorStatus enumeration value:
  *          - SUCCESS: 0
  *          - ERROR: 1
  */
uint32_t RTC_Exit_InitMode(RTC_TypeDef *RTCx)
{
  __IO uint32_t timeout = RTC_INITMODE_TIMEOUT;
  uint32_t status = 0;
  uint32_t tmp = 0U;

  /* Disable initialization mode */
  RTC_Enable_WriteProtection(RTCx);

  /* Wait till RTC is in INIT state and if Time out is reached exit */
  tmp = RTC_Get_Flag(RTC_FLAG_RTOFF);
  while ((timeout != 0U) && (tmp != 1U))
  {
    if ((SysTick->CTRL & SysTick_CTRL_COUNTFLAG_Msk) == (SysTick_CTRL_COUNTFLAG_Msk))
    {
      timeout --;
    }
    tmp = RTC_Get_Flag(RTC_FLAG_RTOFF);
    if (timeout == 0U)
    {
      status = 1;
    }
  }
  return status;
}

/**
  * @brief  Set the Time Counter
  * @param  RTCx RTC Instance
  * @param  TimeCounter this value can be from 0 to 0xFFFFFFFF
  * @retval None.
  */
void RTC_Set_TIMECounter(RTC_TypeDef *RTCx, uint32_t TimeCounter)
{
  /* Enter Initialization mode */
  if (RTC_Enter_InitMode(RTCx) != 1)
  {
    RTC_Set_TIME(RTCx, TimeCounter);
  }
  /* Exit Initialization mode */
  RTC_Exit_InitMode(RTCx);
}

/**
  * @brief  Set Alarm Counter.
  * @param  RTCx RTC Instance
  * @param  AlarmCounter this value can be from 0 to 0xFFFFFFFF
  * @retval None.
  */
void RTC_Set_ALARMCounter(RTC_TypeDef *RTCx, uint32_t AlarmCounter)
{
  /* Enter Initialization mode */
  if (RTC_Enter_InitMode(RTCx) != 1)
  {
    RTC_Set_ALARM(RTCx, AlarmCounter);
  }
  /* Exit Initialization mode */
  RTC_Exit_InitMode(RTCx);
}

/**
  * @brief  Set Asynchronous prescaler factor
  * @param  RTCx RTC Instance
  * @param  AsynchPrescaler Value between Min_Data = 0 and Max_Data = 0xFFFFF
  * @retval None
  */
void RTC_Set_AsynchPrescaler(RTC_TypeDef *RTCx, uint32_t AsynchPrescaler)
{
  MODIFY_REG(RTCx->PRLH, RTC_PRLH_PRL, (AsynchPrescaler >> 16));
  MODIFY_REG(RTCx->PRLL, RTC_PRLL_PRL, (AsynchPrescaler & RTC_PRLL_PRL));
}

/**
  * @brief  Get Asynchronous prescaler factor
  * @param  RTCx RTC Instance
  * @retval Value between Min_Data = 0 and Max_Data = 0xFFFFF
  */
uint32_t RTC_Get_Divider(RTC_TypeDef *RTCx)
{
  register uint16_t Highprescaler = 0, Lowprescaler = 0;
  Highprescaler = READ_REG(RTCx->DIVH & RTC_DIVH_RTC_DIV);
  Lowprescaler  = READ_REG(RTCx->DIVL & RTC_DIVL_RTC_DIV);

  return (((uint32_t) Highprescaler << 16U) | Lowprescaler);
}

/**
  * @brief  Enable the write protection for RTC registers.
  * @param  RTCx RTC Instance
  * @retval None
  */
void RTC_Enable_WriteProtection(RTC_TypeDef *RTCx)
{
  CLEAR_BIT(RTCx->CRL, RTC_CRL_CNF);
}

/**
  * @brief  Disable the write protection for RTC registers.
  * @param  RTCx RTC Instance
  * @retval None
  */
void RTC_Disable_WriteProtection(RTC_TypeDef *RTCx)
{
  SET_BIT(RTCx->CRL, RTC_CRL_CNF);
}

/**
  * @brief  Set time counter in BCD format
  * @note   Bit is write-protected. @ref LL_RTC_DisableWriteProtection function should be called before.
  * @note   It can be written in initialization mode only (@ref RTC_EnterInitMode function)
  * @param  RTCx RTC Instance
  * @param  TimeCounter Value between Min_Data=0x00 and Max_Data=0xFFFFF
  * @retval None
  */
void RTC_Set_TIME(RTC_TypeDef *RTCx, uint32_t TimeCounter)
{
  /* Set RTC COUNTER MSB word */
  WRITE_REG(RTCx->CNTH, (TimeCounter >> 16U));
  /* Set RTC COUNTER LSB word */
  WRITE_REG(RTCx->CNTL, (TimeCounter & RTC_CNTL_RTC_CNT));
}

/**
  * @brief  Get time counter in BCD format
  * @param  RTCx RTC Instance
  * @retval Value between Min_Data = 0 and Max_Data = 0xFFFFF
  */
uint32_t RTC_Get_TIME(RTC_TypeDef *RTCx)
{
  register uint16_t high = 0, low = 0;

  high = READ_REG(RTCx->CNTH & RTC_CNTH_RTC_CNT);
  low  = READ_REG(RTCx->CNTL & RTC_CNTL_RTC_CNT);
  return ((uint32_t)(((uint32_t) high << 16U) | low));
}

/**
  * @brief  Set Alarm Counter
  * @note   Bit is write-protected. @ref RTC_DisableWriteProtection function should be called before.
  * @param  RTCx RTC Instance
  * @param  AlarmCounter Value between Min_Data=0x00 and Max_Data=0xFFFFF
  * @retval None
  */
void RTC_Set_ALARM(RTC_TypeDef *RTCx, uint32_t AlarmCounter)
{
  /* Set RTC COUNTER MSB word */
  WRITE_REG(RTCx->ALRH, (AlarmCounter >> 16));
  /* Set RTC COUNTER LSB word */
  WRITE_REG(RTCx->ALRL, (AlarmCounter & RTC_ALRL_RTC_ALR));
}

/**
  * @brief  Get Alarm Counter
  * @note   Bit is write-protected. @ref RTC_DisableWriteProtection function should be called before.
  * @param  RTCx RTC Instance
  * @retval None
  */
uint32_t RTC_Get_ALARM(RTC_TypeDef *RTCx)
{
  register uint16_t high = 0, low = 0;

  high  = READ_REG(RTCx->ALRH & RTC_ALRH_RTC_ALR);
  low   = READ_REG(RTCx->ALRL & RTC_ALRL_RTC_ALR);

  return (((uint32_t) high << 16U) | low);
}

/**
  * @brief  Enables the specified RTC interrupts.
  * @param  RTC_IT: specifies the RTC interrupts sources to be enabled or disabled.
  *   This parameter can be any combination of the following values:
  *     @arg RTC_IT_OW: Overflow interrupt
  *     @arg RTC_IT_ALR: Alarm interrupt
  *     @arg RTC_IT_SEC: Second interrupt
  * @retval None
  */
void RTC_Enable_IT(uint16_t RTC_IT)
{
  SET_BIT(RTC->CRH, RTC_IT);
}

/**
  * @brief  Disables the specified RTC interrupts.
  * @param  RTC_IT: specifies the RTC interrupts sources to be enabled or disabled.
  *   This parameter can be any combination of the following values:
  *     @arg RTC_IT_OW: Overflow interrupt
  *     @arg RTC_IT_ALR: Alarm interrupt
  *     @arg RTC_IT_SEC: Second interrupt
  * @retval None
  */
void RTC_Disable_IT(uint16_t RTC_IT)
{
  CLEAR_BIT(RTC->CRH, RTC_IT);
}


/**
  * @brief  Checks whether the specified RTC interrupt has occurred or not.
  * @param  RTC_IT: specifies the RTC interrupts sources to check.
  *   This parameter can be one of the following values:
  *     @arg RTC_IT_OW: Overflow interrupt
  *     @arg RTC_IT_ALR: Alarm interrupt
  *     @arg RTC_IT_SEC: Second interrupt
  * @retval The new state of the RTC_IT (SET or RESET).
  */
uint32_t RTC_Get_IT(uint16_t RTC_IT)
{
  return ((READ_BIT(RTC->CRH, RTC_IT) != 0) && (READ_BIT(RTC->CRL, RTC_IT) != 0));
}


/**
  * @brief  Gets the RTC counter value.
  * @param  None
  * @retval RTC counter value.
  */
uint32_t RTC_Get_Counter(void)
{
  return (((uint32_t)RTC->CNTH << 16 ) | RTC->CNTL) ;
}

/**
  * @brief  Sets the RTC counter value.
  * @param  CounterValue: RTC counter new value.
  * @retval None
  */
void RTC_Set_Counter(uint32_t CounterValue)
{ 
  RTC_Disable_WriteProtection(RTC);
  /* Set RTC COUNTER MSB word */
  RTC->CNTH = CounterValue >> 16;
  /* Set RTC COUNTER LSB word */
  RTC->CNTL = (CounterValue & RTC_LSB_MASK);
  RTC_Enable_WriteProtection(RTC);
}

/**
  * @brief  Sets the RTC prescaler value.
  * @param  PrescalerValue: RTC prescaler new value.
  * @retval None
  */
void RTC_Set_Prescaler(uint32_t PrescalerValue)
{ 
  RTC_Disable_WriteProtection(RTC);
  /* Set RTC PRESCALER MSB word */
  RTC->PRLH = (PrescalerValue & PRLH_MSB_MASK) >> 16;
  /* Set RTC PRESCALER LSB word */
  RTC->PRLL = (PrescalerValue & RTC_LSB_MASK);
  RTC_Enable_WriteProtection(RTC);
}

/**
  * @brief  Sets the RTC alarm value.
  * @param  AlarmValue: RTC alarm new value.
  * @retval None
  */
void RTC_Set_Alarm(uint32_t AlarmValue)
{  
  RTC_Disable_WriteProtection(RTC);
  /* Set the ALARM MSB word */
  RTC->ALRH = AlarmValue >> 16;
  /* Set the ALARM LSB word */
  RTC->ALRL = (AlarmValue & RTC_LSB_MASK);
  RTC_Enable_WriteProtection(RTC);
}

/**
  * @brief  Checks whether the specified RTC flag is set or not.
  * @param  RTC_FLAG: specifies the flag to check.
  *   This parameter can be one the following values:
  *     @arg RTC_FLAG_RTOFF: RTC Operation OFF flag
  *     @arg RTC_FLAG_RSF: Registers Synchronized flag
  *     @arg RTC_FLAG_OW: Overflow flag
  *     @arg RTC_FLAG_ALR: Alarm flag
  *     @arg RTC_FLAG_SEC: Second flag
  * @retval The new state of RTC_FLAG (SET or RESET).
  */
uint32_t RTC_Get_Flag(uint16_t RTC_FLAG)
{
  return (READ_BIT(RTC->CRL, RTC_FLAG) == RTC_FLAG);
}

/**
  * @brief  Clears the RTC's pending flags.
  * @param  RTC_FLAG: specifies the flag to clear.
  *   This parameter can be any combination of the following values:
  *     @arg RTC_FLAG_RSF: Registers Synchronized flag. This flag is cleared only after
  *                        an APB reset or an APB Clock stop.
  *     @arg RTC_FLAG_OW: Overflow flag
  *     @arg RTC_FLAG_ALR: Alarm flag
  *     @arg RTC_FLAG_SEC: Second flag
  * @retval None
  */
void RTC_Clear_Flag(uint16_t RTC_FLAG)
{
  /* Clear the corresponding RTC flag */
  CLEAR_BIT(RTC->CRL, RTC_FLAG);
}

/**
  * @brief  Clears the RTC's interrupt pending bits.
  * @param  RTC_IT: specifies the interrupt pending bit to clear.
  *   This parameter can be any combination of the following values:
  *     @arg RTC_IT_OW: Overflow interrupt
  *     @arg RTC_IT_ALR: Alarm interrupt
  *     @arg RTC_IT_SEC: Second interrupt
  * @retval None
  */
void RTC_Clear_ITPendingBit(uint16_t RTC_IT)
{
  /* Clear the corresponding RTC pending bit */
  CLEAR_BIT(RTC->CRL, RTC_IT);
}


/**
  * @brief  Waits until last write operation on RTC registers has finished.
  * @note   This function must be called before any write to RTC registers.
  * @param  None
  * @retval None
  */
void RTC_Wait_LastTask(void)
{
  /* Loop until RTOFF flag is set */
  while ((RTC->CRL & RTC_FLAG_RTOFF) == (uint16_t)RESET)
  {
  }
}

/**
  * @brief  Waits until the RTC registers (RTC_CNT, RTC_ALR and RTC_PRL)
  *   are synchronized with RTC APB clock.
  * @note   This function must be called before any read operation after an APB reset
  *   or an APB clock stop.
  * @param  None
  * @retval None
  */
void RTC_Wait_Synchro(void)
{
  /* Clear RSF flag */
  CLEAR_BIT(RTC->CRL, RTC_FLAG_RSF);
  /* Loop until RSF flag is set */
  while ((RTC->CRL & RTC_FLAG_RSF) == (uint16_t)RESET)
  {
  }
}

